/**
 * LibreSource
 * Copyright (C) 2004-2008 Artenum SARL / INRIA
 * http://www.libresource.org - contact@artenum.com
 *
 * This file is part of the LibreSource software, 
 * which can be used and distributed under license conditions.
 * The license conditions are provided in the LICENSE.TXT file 
 * at the root path of the packaging that enclose this file. 
 * More information can be found at 
 * - http://dev.libresource.org/home/license
 *
 * Initial authors :
 *
 * Guillaume Bort / INRIA
 * Francois Charoy / Universite Nancy 2
 * Julien Forest / Artenum
 * Claude Godart / Universite Henry Poincare
 * Florent Jouille / INRIA
 * Sebastien Jourdain / INRIA / Artenum
 * Yves Lerumeur / Artenum
 * Pascal Molli / Universite Henry Poincare
 * Gerald Oster / INRIA
 * Mariarosa Penzi / Artenum
 * Gerard Sookahet / Artenum
 * Raphael Tani / INRIA
 *
 * Contributors :
 *
 * Stephane Bagnier / Artenum
 * Amadou Dia / Artenum-IUP Blois
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 */
package org.libresource.so6.plugin.wizards;

import org.eclipse.core.resources.IProject;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;

import org.eclipse.jface.dialogs.ErrorDialog;
import org.eclipse.jface.operation.IRunnableWithProgress;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.jface.wizard.IWizardPage;
import org.eclipse.jface.wizard.Wizard;
import org.eclipse.jface.wizard.WizardPage;

import org.eclipse.swt.SWT;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.TabFolder;
import org.eclipse.swt.widgets.TabItem;
import org.eclipse.swt.widgets.Text;

import org.eclipse.team.core.RepositoryProvider;
import org.eclipse.team.ui.IConfigurationWizard;

import org.eclipse.ui.IWorkbench;

import org.libresource.so6.core.Workspace;
import org.libresource.so6.core.WsConnection;
import org.libresource.so6.core.client.ClientIServletImpl;
import org.libresource.so6.core.exec.CreateGenericWsConnection;
import org.libresource.so6.plugin.Plugin;
import org.libresource.so6.plugin.core.So6TeamProvider;
import org.libresource.so6.plugin.core.So6Util;
import org.libresource.so6.plugin.ui.So6Images;

/**
 * @author Guillaume Bort
 * @author Sebastien Jourdain
 * @author Gerald Oster
 */
import java.io.File;

import java.lang.reflect.InvocationTargetException;

import java.net.URI;
import java.net.URL;

import java.util.Properties;


/**
 * DOCUMENT ME!
 *
 * @author $author$
 * @version $Revision: 1.3 $
 */
public class SharingWizard extends Wizard implements IConfigurationWizard {
    /** DOCUMENT ME! */
    protected IProject project;

    /** DOCUMENT ME! */
    protected IWorkbench workbench;

    /** DOCUMENT ME! */
    protected String clientNameValue;

    /** DOCUMENT ME! */
    protected String locationValue;

    /** DOCUMENT ME! */
    protected String nameValue;

    /** DOCUMENT ME! */
    protected String passwordValue;

    /** DOCUMENT ME! */
    protected String serverValue;

    //

    /** DOCUMENT ME! */
    protected String uriValue;

    /** DOCUMENT ME! */
    protected String usernameValue;

    //

    /** DOCUMENT ME! */
    protected WsConnection[] cxs;

    /** DOCUMENT ME! */
    protected boolean connectExistingWorkspace = true;

    /** DOCUMENT ME! */
    protected boolean createDefaultIgnoreFileValue = false;

    /** DOCUMENT ME! */
    protected int selectedConnection;

    /**
     * DOCUMENT ME!
     */
    public void addPages() {
        WizardPage replicaPage = new CreateReplicaPage("createReplica", "Enter So6 queue location informations", So6Images.getSo6WizardImage());
        replicaPage.setDescription("Connect this project to an existing synchronizer");
        addPage(replicaPage);
    }

    /**
     * DOCUMENT ME!
     *
     * @param pageContainer DOCUMENT ME!
     */
    public void createPageControls(Composite pageContainer) {
        super.createPageControls(pageContainer);
        setWindowTitle("Connect the project " + project.getName() + " to a So6 queue");
    }

    /**
     * DOCUMENT ME!
     *
     * @param page DOCUMENT ME!
     *
     * @return DOCUMENT ME!
     */
    public IWizardPage getNextPage(IWizardPage page) {
        return null;
    }

    /**
     * DOCUMENT ME!
     *
     * @return DOCUMENT ME!
     */
    public IWizardPage getStartingPage() {
        return getPage("createReplica");
    }

    /**
     * DOCUMENT ME!
     *
     * @param workbench DOCUMENT ME!
     * @param project DOCUMENT ME!
     */
    public void init(IWorkbench workbench, IProject project) {
        this.workbench = workbench;
        this.project = project;
    }

    /**
     * DOCUMENT ME!
     *
     * @return DOCUMENT ME!
     */
    public boolean performFinish() {
        try {
            String so6Path = null;

            if (connectExistingWorkspace) {
                so6Path = (cxs[selectedConnection].getDataPath() + File.separator + WsConnection.SO6_WSC_FILE).substring(project.getLocation().toString()
                                                                                                                                .length() + 1);
            } else {
                if ((serverValue == null) || (uriValue == null)) {
                    throw new Exception("Invalid uri");
                }

                CreateReplica createReplica = new CreateReplica(uriValue, locationValue, serverValue, usernameValue, passwordValue, nameValue, clientNameValue,
                        createDefaultIgnoreFileValue, serverValue);
                getContainer().run(true, true, createReplica);
                so6Path = (createReplica.connectionPath + File.separator + WsConnection.SO6_WSC_FILE).substring(project.getLocation().toString().length() + 1);
            }

            So6Util.setSo6PropertiesPath(project, so6Path);
            RepositoryProvider.map(project, So6TeamProvider.ID);

            return true;
        } catch (Exception e) {
            e.printStackTrace();
            ErrorDialog.openError(getShell(), "Error in create workspace", e.getMessage(), new Status(IStatus.ERROR, Plugin.ID, IStatus.OK, e.getMessage(), e));

            return false;
        }
    }

    class CreateReplica implements IRunnableWithProgress {
        protected String connectionPath;
        private String clientName;
        private String location;
        private String name;
        private String password;
        private String server;
        private String serverHttpURL;
        private String uri;
        private String user;
        private boolean createDefaultIgnoreFile = false;

        public CreateReplica(String uri, String location, String server, String user, String password, String name, String clientName,
            boolean createDefaultIgnoreFile, String serverHttpURL) {
            this.uri = uri;
            this.location = location;
            this.server = server;
            this.user = user;
            this.password = password;
            this.name = name;
            this.clientName = clientName;
            this.createDefaultIgnoreFile = createDefaultIgnoreFile;
            this.serverHttpURL = serverHttpURL;
        }

        public void run(IProgressMonitor monitor) throws InvocationTargetException, InterruptedException {
            monitor.beginTask("Creating local workspace", 1);

            Properties props = new Properties();

            try {
                if ((serverHttpURL != null) && (serverHttpURL.trim().length() > 0)) {
                    URL serverURL = new URL(serverHttpURL);
                    int port = serverURL.getPort();

                    if (port == -1) {
			if (serverURL.getProtocol().toUpperCase().equals("HTTPS")) {
				port = 443;
			} else {
                        	port = 80;
			}
                    }

                    props.setProperty(ClientIServletImpl.SO6_SERVICE_URL, serverURL.getProtocol() + "://" + serverURL.getHost() + ":" + port + "/ls-so6/so6");
                }

                //
                CreateGenericWsConnection cgwsc = new CreateGenericWsConnection(location, name, clientName, user, password, uri, props);

                if (createDefaultIgnoreFile) {
                    cgwsc.execute(":bin$\n:build$\n\\.class$\n");
                } else {
                    cgwsc.execute();
                }

                connectionPath = cgwsc.getWsc().getDataPath();
                So6Util.refresh(project, monitor);
            } catch (Exception e) {
                throw new CreateReplicaException(e.getMessage());
            }

            monitor.done();
        }
    }

    class CreateReplicaException extends InvocationTargetException {
        private String message;

        public CreateReplicaException(String message) {
            this.message = "Error during the creation of the workspace : " + message;
        }

        public String getMessage() {
            return message;
        }
    }

    class CreateReplicaPage extends WizardPage implements ModifyListener {
        private Button createIgnoreFile;
        private Combo clientName;
        private Combo connections;
        private TabFolder tabFolder;
        private Text name;
        private Text password;
        private Text serveurUrl;
        private Text uri;
        private Text username;

        public CreateReplicaPage(String pageName) {
            super(pageName);
        }

        public CreateReplicaPage(String pageName, String title, ImageDescriptor titleImage) {
            super(pageName, title, titleImage);
        }

        public void createControl(Composite composite) {
            Composite pageContainer = new Composite(composite, SWT.NULL);
            setControl(pageContainer);
            setNeedsProgressMonitor(true);

            GridLayout all = new GridLayout(1, false);
            pageContainer.setLayout(all);
            tabFolder = new TabFolder(pageContainer, SWT.NONE);

            TabItem existingCx = new TabItem(tabFolder, SWT.NONE);
            existingCx.setText("Existing connection");

            //Group existingLocation = new Group(tabFolder, SWT.NONE);
            //existingLocation.setText("Existing connection");
            Composite existingLocation = new Composite(tabFolder, SWT.NONE);
            GridLayout somelayout = new GridLayout(1, false);
            existingLocation.setLayout(somelayout);
            connections = new Combo(existingLocation, SWT.NONE);
            connections.addModifyListener(this);

            if ((new File(project.getLocation().toString() + "/.so6")).exists()) {
                try {
                    Workspace tmpws = new Workspace(project.getLocation().toString());

                    //System.out.println(project.getLocation().toString());
                    cxs = tmpws.listConnections();

                    for (int i = 0; i < cxs.length; i++) {
                        connections.add(cxs[i].getWsName() + " [" + cxs[i].getDataPath() + "]");
                    }

                    if (cxs.length == 0) {
                        connections.setEnabled(false);
                    }
                } catch (Exception e) {
                    //e.printStackTrace();
                    connections.setEnabled(false);
                }

                connections.setText("Please choose an existing connection");
            }

            existingCx.setControl(existingLocation);

            // queue location
            TabItem newCx = new TabItem(tabFolder, SWT.NONE);
            newCx.setText("Create a new connection");

            //Group queueLocation = new Group(tabFolder, SWT.NONE);
            //queueLocation.setText("replica configuration");
            Composite queueLocation = new Composite(tabFolder, SWT.NONE);
            GridLayout layout = new GridLayout(2, false);
            queueLocation.setLayout(layout);

            Label nameLabel = new Label(queueLocation, SWT.NONE);
            nameLabel.setText("Workspace name : ");
            name = new Text(queueLocation, SWT.SINGLE | SWT.BORDER);
            name.addModifyListener(this);

            // Synchronizer uri
            GridData gd = new GridData();
            gd.widthHint = 300;
            name.setLayoutData(gd);

            Label queueURI = new Label(queueLocation, SWT.NONE);
            queueURI.setText("Synchronizer URI : ");
            uri = new Text(queueLocation, SWT.SINGLE | SWT.BORDER);
            uri.addModifyListener(this);

            // user name
            gd = new GridData();
            gd.widthHint = 300;
            uri.setLayoutData(gd);

            Label usernameLabel = new Label(queueLocation, SWT.NONE);
            usernameLabel.setText("Username : ");
            gd = new GridData();
            gd.widthHint = 300;
            username = new Text(queueLocation, SWT.SINGLE | SWT.BORDER);
            username.addModifyListener(this);
            username.setLayoutData(gd);

            // Password
            Label passwordLabel = new Label(queueLocation, SWT.NONE);
            passwordLabel.setText("Password : ");
            gd = new GridData();
            gd.widthHint = 300;
            password = new Text(queueLocation, SWT.SINGLE | SWT.BORDER);
            password.addModifyListener(this);
            password.setLayoutData(gd);
            password.setEchoChar('*');

            // Server host
            gd = new GridData();
            gd.widthHint = 300;
            uri.setLayoutData(gd);

            Label serveurUrlLabel = new Label(queueLocation, SWT.NONE);
            serveurUrlLabel.setText("Server address : ");
            gd = new GridData();
            gd.widthHint = 300;
            serveurUrl = new Text(queueLocation, SWT.SINGLE | SWT.BORDER);
            serveurUrl.addModifyListener(this);
            serveurUrl.setLayoutData(gd);

            // Client Name
            gd = new GridData();
            gd.widthHint = 300;
            uri.setLayoutData(gd);

            Label clientNameLabel = new Label(queueLocation, SWT.NONE);
            clientNameLabel.setText("Client type : ");
            gd = new GridData();
            gd.widthHint = 300;
            clientName = new Combo(queueLocation, SWT.SINGLE | SWT.BORDER);
            clientName.add("Local client");
            clientName.add("LibreSource client");
            clientName.addModifyListener(this);
            clientName.setLayoutData(gd);
            clientName.select(1);

            // Create ignore file
            /*
               gd = new GridData();
               gd.widthHint = 300;
               createIgnoreFile = new Button(queueLocation, SWT.SINGLE | SWT.BORDER);
               createIgnoreFile.setText("Create default ignore file");
               createIgnoreFile.setLayoutData(gd);
             */
            // 
            newCx.setControl(queueLocation);
        }

        public void modifyText(ModifyEvent e) {
            try {
                connectExistingWorkspace = this.tabFolder.getSelectionIndex() == 0;
                uriValue = new URI(this.uri.getText()).getPath();
                usernameValue = this.username.getText();
                passwordValue = this.password.getText();
                serverValue = this.serveurUrl.getText();
                locationValue = project.getLocation().toString();
                nameValue = this.name.getText();

                if (this.clientName.getSelectionIndex() == 0) {
                    clientNameValue = CreateGenericWsConnection.FS_CLIENT_NAME;
                } else {
                    clientNameValue = CreateGenericWsConnection.LS_CLIENT_NAME;
                }

                createDefaultIgnoreFileValue = false;
                selectedConnection = connections.getSelectionIndex();
            } catch (Exception e1) {
                //
            }
        }
    }
}
