/**
 * LibreSource Community
 * Copyright (C) 2004-2006 Artenum SARL / INRIA
 * http://www.libresource.org - contact@libresource.org
 *
 * This software is not a free software; you can modify it under the
 * LibreSource Enterprise user license but you can't redistribute it.
 * See licenses details in LSE-user-license.txt 
 *
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 */
package org.libresource.web.controllers.membership;

import org.libresource.Libresource;

import org.libresource.kernel.LibresourceSecurityException;
import org.libresource.kernel.interfaces.KernelService;

import org.libresource.membership.ejb.model.ProfileResourceValue;
import org.libresource.membership.interfaces.MembershipService;

import org.libresource.web.Controller;
import org.libresource.web.config.Config;

import java.net.URI;

import java.util.Arrays;
import java.util.TimeZone;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;


public class EditProfileController implements Controller {
    public Object process(URI uri, HttpServletRequest request, HttpServletResponse response)
        throws Exception {
        if (request.getParameter("cancel") != null) {
            return uri;
        }

        KernelService kernelService = (KernelService) Libresource.getService("Kernel");

        if (!kernelService.checkSecurity(uri, "Kernel:UPDATE")) {
            throw new LibresourceSecurityException(uri, "Kernel:UPDATE");
        }

        MembershipService membershipService = (MembershipService) Libresource.getService("Membership");
        ProfileResourceValue user = membershipService.getProfile(uri);

        // init form
        if (request.getParameter("name") == null) {
            // lists
            request.setAttribute("skins", Config.getInstance(null).listSkins());
            request.setAttribute("langs", Config.getInstance(null).listLangs());

            String[] timeZones = TimeZone.getAvailableIDs();
            Arrays.sort(timeZones);
            request.setAttribute("timeZones", timeZones);
            request.setAttribute("units", Config.getInstance(null).listUnits());

            request.setAttribute("login", user.getId());
            request.setAttribute("name", user.getFullName());
            request.setAttribute("email", user.getEmail());
            request.setAttribute("jabber", user.getJabberId());

            return "/pages/modules/membership/editProfile.jsp";
        }

        // process new values
        String name = request.getParameter("name");

        if (name.trim().equals("")) {
            name = user.getId();
        }

        membershipService.editProfile(uri, name, request.getParameter("email"), request.getParameter("jabber"));

        // lang
        String lang = request.getParameter("lang");

        if (lang != null) {
            membershipService.setUserInfo(uri, "libresource-web.lang", lang);
        }

        // skin
        String skin = request.getParameter("skin");

        if (skin != null) {
            membershipService.setUserInfo(uri, "libresource-web.skin", skin);
        }

        // timeZone
        String timeZone = request.getParameter("timeZone");

        if (timeZone != null) {
            membershipService.setUserInfo(uri, "libresource-web.time-zone", timeZone);
        }

        // unit
        String unit = request.getParameter("unit");

        if (unit != null) {
            membershipService.setUserInfo(uri, "libresource-web.unit", unit);
        }

        // authentification part
        String newPass1 = request.getParameter("newpass1");
        String newPass2 = request.getParameter("newpass2");

        if ((newPass1 != null) && (newPass2 != null) && (newPass1.trim().length() > 0) && (newPass2.trim().length() > 0)) {
            if (newPass1.equals(newPass2)) {
                membershipService.editCasUser(user.getId(), newPass1);
            }
        }

        // clean cache
        request.getSession().removeAttribute("connectedUserCache");

        return uri;
    }
}
