/**
 * LibreSource
 * Copyright (C) 2004-2008 Artenum SARL / INRIA
 * http://www.libresource.org - contact@artenum.com
 *
 * This file is part of the LibreSource software, 
 * which can be used and distributed under license conditions.
 * The license conditions are provided in the LICENSE.TXT file 
 * at the root path of the packaging that enclose this file. 
 * More information can be found at 
 * - http://dev.libresource.org/home/license
 *
 * Initial authors :
 *
 * Guillaume Bort / INRIA
 * Francois Charoy / Universite Nancy 2
 * Julien Forest / Artenum
 * Claude Godart / Universite Henry Poincare
 * Florent Jouille / INRIA
 * Sebastien Jourdain / INRIA / Artenum
 * Yves Lerumeur / Artenum
 * Pascal Molli / Universite Henry Poincare
 * Gerald Oster / INRIA
 * Mariarosa Penzi / Artenum
 * Gerard Sookahet / Artenum
 * Raphael Tani / INRIA
 *
 * Contributors :
 *
 * Stephane Bagnier / Artenum
 * Amadou Dia / Artenum-IUP Blois
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 */
package org.libresource.web.controllers.so6;

import org.libresource.Libresource;

import org.libresource.kernel.KernelConstants;
import org.libresource.kernel.interfaces.KernelService;

import org.libresource.so6.core.client.ClientIServletImpl;
import org.libresource.so6.server.ls.LibresourceSynchronizerConstants;
import org.libresource.so6.server.ls.ejb.model.WsConnectionResourceValue;
import org.libresource.so6.server.ls.interfaces.LibresourceSynchronizerService;

import org.libresource.web.Controller;

import java.io.File;

import java.net.URI;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;


public class UpdateLocalReplicaController implements Controller {
    public Object process(URI uri, HttpServletRequest request, HttpServletResponse response)
        throws Exception {
        LibresourceSynchronizerService libresourceSynchronizerService = (LibresourceSynchronizerService) Libresource.getService(LibresourceSynchronizerConstants.SERVICE);
        WsConnectionResourceValue value = libresourceSynchronizerService.getWorkspaceConnection(uri);

        // Otherwise: if Unix server/Windows client -> file with no parent !!!
        File wscPath = new File(value.getWsPath().replaceAll("\\\\", "/"));

        //      take care of the security
        String protocol = "http";
        String clientName = ClientIServletImpl.class.getName();

        protocol = request.getScheme();

        // SO6 default
        request.setAttribute("queueId", libresourceSynchronizerService.getWorkspaceConnectionSynchronizer(uri).getUri().getPath());
        request.setAttribute("login", "");
        request.setAttribute("password", "");
        request.setAttribute("connectionId", uri);

        // WsConnection default
        request.setAttribute("wscPath", wscPath.getPath());
        request.setAttribute("dataPath", wscPath.getParent());
        request.setAttribute("basePath", wscPath.getParentFile().getParentFile().getParent());
        request.setAttribute("clientName", clientName);
        request.setAttribute("wscName", value.getName());

        // Client specific props
        // servlet -->
        request.setAttribute("soapEndpoint",
            protocol + "://" + request.getServerName() + ":" + request.getServerPort() + request.getContextPath() + "/ls-so6/services/so6");

        // soap -->
        request.setAttribute("servletEndpoint",
            protocol + "://" + request.getServerName() + ":" + request.getServerPort() + request.getContextPath() + "/ls-so6/so6");

        KernelService kernelService = (KernelService) Libresource.getService(KernelConstants.SERVICE);
        request.setAttribute("encoding", libresourceSynchronizerService.getSynchronizer(kernelService.getParent(uri)).getFileEncoding());

        return "/pages/modules/so6/jnlp/updateLocalReplicaData.jsp";
    }
}
