/**
 * LibreSource Community
 * Copyright (C) 2004-2006 Artenum SARL / INRIA
 * http://www.libresource.org - contact@libresource.org
 *
 * This software is not a free software; you can modify it under the
 * LibreSource Enterprise user license but you can't redistribute it.
 * See licenses details in LSE-user-license.txt 
 *
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 */
package org.libresource.web.servlets;

import edu.yale.its.tp.cas.client.filter.CASFilter;

import org.libresource.Libresource;

import org.libresource.kernel.interfaces.KernelService;

import org.libresource.membership.interfaces.MembershipService;

import org.objectweb.jonas.security.auth.callback.NoInputCallbackHandler;

import java.net.URI;

import java.util.HashMap;

import javax.security.auth.callback.CallbackHandler;
import javax.security.auth.login.LoginContext;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;


public class CASLogin extends BaseServlet {
    protected void process(HttpServletRequest request, HttpServletResponse response)
        throws Exception {
        KernelService kernelService = (KernelService) Libresource.getService("Kernel");
        MembershipService membershipService = (MembershipService) Libresource.getService("Membership");

        // get user from cas
        String username = (String) request.getSession().getAttribute(CASFilter.CAS_FILTER_USER);

        // login to libresource
        CallbackHandler handler = new NoInputCallbackHandler(username, "null");
        LoginContext loginContext = new LoginContext("libresource", handler);
        loginContext.login();
        kernelService.getConnectedResource();

        // check if first login
        try {
            membershipService.getProfile();
        } catch (Exception e) {
            // create a default profile
            kernelService.systemCreateURI(new URI(membershipService.getUsersRootURI() + "/" + username));
            membershipService.systemCreateProfile(username, username, "", "", new HashMap());
        }

        // init session
        loginContext.logout();
        request.getSession().invalidate();
        request.getSession().setAttribute(CASFilter.CAS_FILTER_USER, username);
        request.getSession().setAttribute("security_username", username);

        String redirect = request.getParameter("loginRedirectURL");

        if (redirect != null) {
            response.sendRedirect(redirect);

            return;
        }

        response.sendRedirect(request.getContextPath());
    }
}
