/**
 * LibreSource
 * Copyright (C) 2004-2008 Artenum SARL / INRIA
 * http://www.libresource.org - contact@artenum.com
 *
 * This file is part of the LibreSource software, 
 * which can be used and distributed under license conditions.
 * The license conditions are provided in the LICENSE.TXT file 
 * at the root path of the packaging that enclose this file. 
 * More information can be found at 
 * - http://dev.libresource.org/home/license
 *
 * Initial authors :
 *
 * Guillaume Bort / INRIA
 * Francois Charoy / Universite Nancy 2
 * Julien Forest / Artenum
 * Claude Godart / Universite Henry Poincare
 * Florent Jouille / INRIA
 * Sebastien Jourdain / INRIA / Artenum
 * Yves Lerumeur / Artenum
 * Pascal Molli / Universite Henry Poincare
 * Gerald Oster / INRIA
 * Mariarosa Penzi / Artenum
 * Gerard Sookahet / Artenum
 * Raphael Tani / INRIA
 *
 * Contributors :
 *
 * Stephane Bagnier / Artenum
 * Amadou Dia / Artenum-IUP Blois
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 */
package org.libresource.web.servlets;

import org.apache.commons.fileupload.FileItem;
import org.apache.commons.fileupload.FileItemFactory;
import org.apache.commons.fileupload.disk.DiskFileItemFactory;
import org.apache.commons.fileupload.servlet.ServletFileUpload;
import org.apache.commons.fileupload.servlet.ServletRequestContext;

import org.libresource.Libresource;

import org.libresource.core.FileDataImpl;

import org.libresource.kernel.KernelConstants;
import org.libresource.kernel.interfaces.KernelService;

import org.libresource.web.WebImportExportLogger;
import org.libresource.web.config.Config;
import org.libresource.web.taglibs.JspFunctions;

import org.libresource.xml.LibresourceImportExport;

import java.io.File;
import java.io.FileOutputStream;
import java.io.InputStream;

import java.net.URI;
import java.net.URL;

import java.util.Iterator;
import java.util.List;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;


public class Import extends BaseServlet {
    protected void process(HttpServletRequest request, HttpServletResponse response)
        throws Exception {
        if (!ServletFileUpload.isMultipartContent(new ServletRequestContext(request)) && (request.getParameter("defaultBootstrap") == null)) {
            if (request.getParameter("uri") != null) {
                request.setAttribute("uri", request.getParameter("uri"));
            }

            request.getRequestDispatcher("/pages/import.jsp").forward(request, response);

            return;
        }

        // Common vars
        boolean createProfile = false;
        FileDataImpl fileData = null;
        String uri = null;
        String fileName = "defaultBootstrap.xml";
        long size = 0;

        // Case of custom bootstrap
        if (ServletFileUpload.isMultipartContent(new ServletRequestContext(request))) {
            // Create a new file upload handler
            FileItemFactory factory = new DiskFileItemFactory();
            ServletFileUpload upload = new ServletFileUpload(factory);
            List items = upload.parseRequest(request);

            // Process the uploaded items
            Iterator iter = items.iterator();

            while (iter.hasNext()) {
                FileItem item = (FileItem) iter.next();

                if (item.isFormField()) {
                    if (item.getFieldName().equals("uri")) {
                        uri = item.getString();
                    }

                    if (item.getFieldName().equals("createProfile")) {
                        createProfile = item.getString().equals("on");
                    }
                } else {
                    if (item.getFieldName().equals("file")) {
                        if (item.getSize() > 0) {
                            File temp = File.createTempFile("file", "import", Config.getInstance(null).getLsWebBuffer());
                            item.write(temp);
                            fileData = new FileDataImpl(temp);
                            fileName = item.getName();
                            size = item.getSize();
                        }
                    }
                }
            }
        } else if (request.getParameter("defaultBootstrap") != null) {
            // Case of default bootstrap
            uri = "";
            createProfile = true;

            // Build file data
            File temp = File.createTempFile("file", "import", Config.getInstance(null).getLsWebBuffer());
            InputStream is = new URL(org.libresource.Libresource.getLibresourceConfiguration("kernel.serverUrl") + "/static/download/Bootstrap.xml").openConnection()
                                                                                                                                                    .getInputStream();
            FileOutputStream os = new FileOutputStream(temp);
            int readCount = -1;
            byte[] buffer = new byte[1024 * 1024 * 2];
            size = 0;

            while ((readCount = is.read(buffer)) > 0) {
                os.write(buffer, 0, readCount);
                size += readCount;
            }

            os.close();
            fileData = new FileDataImpl(temp);
        }

        try {
            response.setContentType("text/html");

            URI connectedUser = ((KernelService) Libresource.getService(KernelConstants.SERVICE)).getConnectedResource();
            response.getWriter().println("<html><head><base href=\"" + request.getScheme() + "://" + request.getServerName() + ":" + request.getServerPort() +
                request.getContextPath() + "/\"/><link rel=\"stylesheet\" type=\"text/css\" href=\"" + JspFunctions.getSkin(connectedUser).getCss() +
                "\" media=\"screen\"/></head><body><h1 class=\"heading-1\">Importing " + fileName + " (" + size + " o.) at " + uri + " (you are " +
                connectedUser + ") </h1><ul>");

            System.out.println("IMPORT, createProfile = " + createProfile);
            new LibresourceImportExport(createProfile).importXML(new URI(uri), fileData.getInputStream(), new WebImportExportLogger(response));

            response.getWriter().println("</ul><div><a href=\"" + ((new URI(uri).getPath().length() > 0) ? new URI(uri).getPath().substring(1) : "") +
                "?action=indexation\">Go to " + uri + "</a></div></body></html>");

            for (int i = 0; i < 100; i++)
                response.getWriter().println("                                 ");
        } catch (Exception e) {
            response.getWriter().println("<pre class=\"error\">");

            // e.printStackTrace(new PrintWriter(response.getWriter()));
            StringBuffer buffer = new StringBuffer();
            buffer.append(e.getMessage() + "<br>");

            Throwable cause = null;

            while ((cause = e.getCause()) != null) {
                buffer.append("<i>Caused by :</i> " + cause.getMessage() + "<br>");
                e = (Exception) cause;
            }

            response.getWriter().println(buffer.toString());
            response.getWriter().println("</pre>");
            response.getWriter().println("<a href=\"/\">Return to Platform Home</a></body></html>");
        }

        response.getWriter().flush();
        response.flushBuffer();
        response.getWriter().close();
    }
}
