/**
 * LibreSource Community
 * Copyright (C) 2004-2006 Artenum SARL / INRIA
 * http://www.libresource.org - contact@libresource.org
 *
 * This software is not a free software; you can modify it under the
 * LibreSource Enterprise user license but you can't redistribute it.
 * See licenses details in LSE-user-license.txt 
 *
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 */
package org.libresource.web.servlets;

import edu.yale.its.tp.cas.client.filter.CASFilter;

import org.libresource.Libresource;

import org.libresource.kernel.interfaces.KernelService;

import org.objectweb.jonas.security.auth.callback.NoInputCallbackHandler;

import java.io.BufferedOutputStream;
import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.io.OutputStreamWriter;

import java.net.HttpURLConnection;
import java.net.URL;
import java.net.URLEncoder;

import javax.security.auth.callback.CallbackHandler;
import javax.security.auth.login.LoginContext;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;


public class Login extends BaseServlet {
    private KernelService kernelService;

    protected void process(HttpServletRequest request, HttpServletResponse response)
        throws Exception {
        if (request.getParameter("username") == null) {
            request.getRequestDispatcher("/pages/login.jsp").forward(request, response);

            return;
        }

        String username = request.getParameter("username");
        String password = request.getParameter("password");

        // //////////////////////////////////////////////////////////////////
        // automatic login on cas
        // get url complete path
        URL initUrl = getAuthUrl();

        // create first post request (init lt)
        HttpURLConnection init = (HttpURLConnection) initUrl.openConnection();
        init.setUseCaches(false);
        init.setRequestProperty("Connection", "Keep-Alive");
        init.setRequestMethod("POST");

        // read response to set lt
        BufferedReader reader = new BufferedReader(new InputStreamReader(init.getInputStream()));
        String lt = null;
        String s;

        // tout ca n'est qu'un vieu hack moisi
        while ((s = reader.readLine()) != null) {
            if (s.trim().startsWith("<input type=\"hidden\"")) {
                lt = s.substring(s.indexOf("value=") + 7, s.lastIndexOf('"'));
            }
        }

        // try to connect with the user params
        URL loginUrl = getAuthUrl();

        // create second post request (login)
        HttpURLConnection login = (HttpURLConnection) loginUrl.openConnection();
        login.setUseCaches(false);
        login.setRequestProperty("Connection", "Keep-Alive");
        login.setRequestMethod("POST");
        login.setDoOutput(true);

        OutputStreamWriter osw = new OutputStreamWriter(new BufferedOutputStream(login.getOutputStream()), "ASCII");
        String postparameter = URLEncoder.encode("username", "UTF-8") + "=" + URLEncoder.encode(username, "UTF-8");

        // System.out.println(postparameter);
        osw.write(postparameter); // POST occurs here for name value pairs
        postparameter = "&" + URLEncoder.encode("password", "UTF-8") + "=" + URLEncoder.encode(password, "UTF-8");

        // System.out.println(postparameter);
        osw.write(postparameter); // POST occurs here for name value pairs
        postparameter = "&" + URLEncoder.encode("lt", "UTF-8") + "=" + URLEncoder.encode(lt, "UTF-8");

        // System.out.println(postparameter);
        osw.write(postparameter); // POST occurs here for name value pairs
        osw.close();

        if (login.getHeaderField("Set-Cookie") == null) {
            // user not connected
            request.setAttribute("error", "Login failed !");
            request.getRequestDispatcher("/pages/login.jsp").forward(request, response);

            return;

            // username = null;
        }

        // end cas
        // //////////////////////////////////////////////////////////////////
        CallbackHandler handler = new NoInputCallbackHandler(username, password);
        LoginContext loginContext = new LoginContext("libresource", handler);

        try {
            loginContext.login();
            getKernelService().getConnectedResource();
        } catch (Exception e) {
            request.setAttribute("error", e.getMessage());
            request.getRequestDispatcher("/pages/login.jsp").forward(request, response);

            return;
        }

        loginContext.logout();

        request.getSession().invalidate();
        request.getSession().setAttribute(CASFilter.CAS_FILTER_USER, username);
        request.getSession().setAttribute("security_username", username);

        // request.getSession().setAttribute("security_password", password);
        String redirect = (request.getParameter("loginRedirectURL").endsWith("CreateUser")) ? ("users/" + request.getParameter("username"))
                                                                                            : request.getParameter("loginRedirectURL");
        response.sendRedirect(redirect);
    }

    private URL getAuthUrl() throws Exception {
        String authUrl = getKernelService().getAuthUrl();

        return new URL(authUrl + "/login");
    }

    private KernelService getKernelService() throws Exception {
        if (kernelService == null) {
            kernelService = (KernelService) Libresource.getService("Kernel");
        }

        return kernelService;
    }
}
