/**
 * LibreSource Community
 * Copyright (C) 2004-2006 Artenum SARL / INRIA
 * http://www.libresource.org - contact@libresource.org
 *
 * This software is not a free software; you can modify it under the
 * LibreSource Enterprise user license but you can't redistribute it.
 * See licenses details in LSE-user-license.txt 
 *
 * Initial authors :
 *
 * Guillaume Bort / INRIA
 * Francois Charoy / Universite Nancy 2
 * Julien Forest / Artenum
 * Claude Godart / Universite Henry Poincare
 * Florent Jouille / INRIA
 * Sebastien Jourdain / INRIA / Artenum
 * Yves Lerumeur / Artenum
 * Pascal Molli / Universite Henry Poincare
 * Gerald Oster / INRIA
 * Mariarosa Penzi / Artenum
 * Gerard Sookahet / Artenum
 * Raphael Tani / INRIA
 *
 * Contributors :
 *
 * Stephane Bagnier / Artenum
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 */
package org.libresource.so6.web.filter;

import org.apache.axis.encoding.Base64;
import org.apache.axis.transport.http.HTTPConstants;

import org.libresource.Libresource;

import org.libresource.kernel.interfaces.KernelService;

import org.libresource.so6.web.servlet.So6Actions;

import org.objectweb.jonas.security.auth.callback.NoInputCallbackHandler;

import java.io.BufferedOutputStream;
import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.OutputStreamWriter;

import java.net.HttpURLConnection;
import java.net.URL;
import java.net.URLEncoder;

import javax.security.auth.callback.CallbackHandler;
import javax.security.auth.login.LoginContext;

import javax.servlet.Filter;
import javax.servlet.FilterChain;
import javax.servlet.FilterConfig;
import javax.servlet.ServletException;
import javax.servlet.ServletRequest;
import javax.servlet.ServletResponse;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;


public class SecurityFilter implements Filter {
    protected FilterConfig filterConfig;
    private KernelService kernelService;

    // Called once when this filter is instantiated. If mapped to
    // j_security_check, called very first time j_security_check is invoked.
    public void init(FilterConfig filterConfig) throws ServletException {
        this.filterConfig = filterConfig;
    }

    public void destroy() {
        this.filterConfig = null;
    }

    // Called for every request that is mapped to this filter. If mapped to
    // j_security_check, called for every j_security_check action
    public void doFilter(ServletRequest request, ServletResponse response, FilterChain chain)
        throws IOException, ServletException {
        try {
            HttpServletResponse httpResponse = (HttpServletResponse) response;
            HttpServletRequest httpRequest = (HttpServletRequest) request;

            // Retreive login/password
            String username = httpRequest.getHeader("login");
            String password = httpRequest.getHeader("password");
            String basicLogin = httpRequest.getHeader(HTTPConstants.HEADER_AUTHORIZATION);

            //
            if ((basicLogin != null) && (basicLogin.indexOf("Basic") != -1)) {
                String loginPassword = new String(Base64.decode(basicLogin.substring("Basic ".length()).trim()));
                username = loginPassword.substring(0, loginPassword.indexOf(":"));
                password = loginPassword.substring(loginPassword.indexOf(":") + 1);
            }

            //System.out.println("So6 security filter | login: " + username);
            //System.out.println("So6 security filter | password: " + password);
            // //////////////////////////////////////////////////////////////////
            // automatic login on cas
            if ((username != null) && (password != null)) {
                // get url complete path
                URL initUrl = getAuthUrl();

                // create first post request (init lt)
                HttpURLConnection init = (HttpURLConnection) initUrl.openConnection();
                init.setUseCaches(false);
                init.setRequestProperty("Connection", "Keep-Alive");
                init.setRequestMethod("POST");

                // read response to set lt
                BufferedReader reader = new BufferedReader(new InputStreamReader(init.getInputStream()));
                String lt = null;
                String s;

                // tout ca n'est qu'un vieu hack moisi
                while ((s = reader.readLine()) != null) {
                    if (s.trim().startsWith("<input type=\"hidden\"")) {
                        lt = s.substring(s.indexOf("value=") + 7, s.lastIndexOf('"'));
                    }
                }

                // try to connect with the user params
                URL loginUrl = getAuthUrl();

                // create second post request (login)
                HttpURLConnection login = (HttpURLConnection) loginUrl.openConnection();
                login.setUseCaches(false);
                login.setRequestProperty("Connection", "Keep-Alive");
                login.setRequestMethod("POST");
                login.setDoOutput(true);

                OutputStreamWriter osw = new OutputStreamWriter(new BufferedOutputStream(login.getOutputStream()), "ASCII");
                String postparameter = URLEncoder.encode("username", "UTF-8") + "=" + URLEncoder.encode(username, "UTF-8");
                osw.write(postparameter); // POST occurs here for name value pairs
                postparameter = "&" + URLEncoder.encode("password", "UTF-8") + "=" + URLEncoder.encode(password, "UTF-8");
                osw.write(postparameter); // POST occurs here for name value pairs
                postparameter = "&" + URLEncoder.encode("lt", "UTF-8") + "=" + URLEncoder.encode(lt, "UTF-8");
                osw.write(postparameter); // POST occurs here for name value pairs
                osw.close();

                if (login.getHeaderField("Set-Cookie") == null) {
                    // user not connected
                    username = null;
                }
            }

            // end cas
            // //////////////////////////////////////////////////////////////////
            if (username == null) {
                chain.doFilter(httpRequest, httpResponse);
            } else {
                // Login
                CallbackHandler handler = new NoInputCallbackHandler(username, password);
                LoginContext loginContext = new LoginContext("libresource", handler);

                //
                loginContext.login();

                //
                chain.doFilter(httpRequest, httpResponse);

                //
                loginContext.logout();
            }
        } catch (Exception e) {
            e.printStackTrace();
            ((HttpServletResponse) response).sendError(So6Actions.STATUS_CODE_AUTH_ERROR);
        }
    }

    private URL getAuthUrl() throws Exception {
        String authUrl = getKernelService().getAuthUrl();

        return new URL(authUrl + "/login");
    }

    private KernelService getKernelService() throws Exception {
        if (kernelService == null) {
            kernelService = (KernelService) Libresource.getService("Kernel");
        }

        return kernelService;
    }
}
